package tdanford.sunlight.cgp;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.LinkedList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class CGPParser {
	
	private LinkedList<MARC21Record> records;

	public CGPParser(File cgpFile) throws SAXException, IOException {
		records = new LinkedList<MARC21Record>();
		
		try { 
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			MARCXMLHandler handler = new MARCXMLHandler();
			parser.parse(cgpFile, handler);

			records.addAll(handler.getRecords());
		} catch(ParserConfigurationException e) { 
			throw new IllegalStateException(e);
		} catch(IOException e) {
			throw new IOException(String.format("file: %s", cgpFile.getAbsolutePath()), e);
		} catch(SAXParseException e) {
			throw new SAXException(String.format("file: %s", cgpFile.getAbsolutePath()));
		} catch(SAXException e) {
			throw new SAXException(String.format("file: %s", cgpFile.getAbsolutePath()), e);
		}
	}
	
	public CGPParser(Collection<File> cgpFiles) throws SAXException, IOException { 
		records = new LinkedList<MARC21Record>();
		for(File f : cgpFiles) { 
			records.addAll(new CGPParser(f).getRecords());
		}
	}
	
	public CGPParser(SunlightCGPDirectory finder) throws SAXException, IOException { 
		this(finder.collectLatestFiles());
	}
	
	public Collection<MARC21Record> getRecords() { 
		return records;
	}
}
