package tdanford.sunlight.cgp;

import java.util.LinkedHashMap;
import java.util.Map;
import java.lang.reflect.*;

public class CharValueSet {

	public String fieldName;
	public int fieldOffset;
	public Map<Character,String> descriptions; 
	
	public CharValueSet() { 
		descriptions = new LinkedHashMap<Character,String>();
		
		parse();
	}
	
	public String interpret(Object obj) { 
		Class cls = obj.getClass();
		for(Field f : cls.getFields()) { 
			int mod = f.getModifiers();
			if(Modifier.isPublic(mod) && !Modifier.isStatic(mod)) { 
				Class type = f.getType();
				if(type.equals(Character.class)) { 
					ParseField leaderField = f.getAnnotation(ParseField.class);
					if(leaderField != null && leaderField.offset() == fieldOffset) { 
						try {
							Character value = (Character) f.get(obj);
							if(descriptions.containsKey(value)) { 
								return descriptions.get(value);
							} else { 
								throw new IllegalStateException(String.format(
										"Illegal value %c for %s", value, fieldName));
							}
						
						} catch (IllegalAccessException e) {
							throw new IllegalStateException(e);
						}
						
					}
				}
			}
		}
		
		return null;
	}
	
	private void parse() {
		
		CharField leaderField = getClass().getAnnotation(CharField.class);
		fieldName = leaderField.name();
		fieldOffset = leaderField.offset();
		if(!leaderField.type().equals(Character.class)) { 
			throw new IllegalStateException(String.format("%s %s", leaderField.name(), leaderField.type().getSimpleName()));
		}
		
		Field[] fs = getClass().getFields();
		for(Field f : fs) { 
			int mod = f.getModifiers();
			if(Modifier.isPublic(mod) && Modifier.isStatic(mod)) { 
				Class type = f.getType();
				if(type.equals(String.class)) { 
					
					try {
						String description = (String) f.get(this);
						CharValue charValue = f.getAnnotation(CharValue.class);
						if(charValue != null) { 
							Character key = charValue.value();
							descriptions.put(key, description);
						}
					
					} catch (IllegalAccessException e) {
						throw new IllegalStateException(e);
					}
				}
			}
		}
	}
}
