package tdanford.sunlight.cgp;

import java.lang.reflect.*;

public class LeaderRecord {

	public LeaderRecord(MARC21Record rec) { 
		Class cls = getClass();
		char[] leader = rec.leader.content.toCharArray();
		
		for(Field f : cls.getFields()) { 
			int mod = f.getModifiers();
			if(Modifier.isPublic(mod) && !Modifier.isStatic(mod)) { 
				Class type = f.getType();
				if(type.equals(Character.class)) { 
					ParseField charField = f.getAnnotation(ParseField.class);
					if(charField != null && charField.type().equals(type)) {
						
						int offset = charField.offset();
						Character leaderValue = leader[offset];
						try {
							f.set(this, leaderValue);
						} catch (IllegalAccessException e) {
							throw new IllegalStateException(e);
						}
					}
				}
			}
		}
	}
	
	public static boolean isSubclass(Class c1, Class c2) { 
		return c2.isAssignableFrom(c1);
	}
	
	public String interpret() { 
		StringBuilder sb = new StringBuilder();

		Class cls = getClass();
		
		for(Field f : cls.getFields()) { 
			int mod = f.getModifiers();
			if(Modifier.isPublic(mod) && !Modifier.isStatic(mod)) { 
				Class type = f.getType();
				if(type.equals(Character.class)) { 
					ParseField parseField = f.getAnnotation(ParseField.class);
					if(parseField != null) {
						
						Class interpClass = parseField.interp();
						if(isSubclass(interpClass, CharValueSet.class)) {
							
							CharField charField = (CharField)interpClass.getAnnotation(CharField.class);

							try { 
								CharValueSet interpreter = (CharValueSet)(interpClass.newInstance());

								if(sb.length() > 0) { sb.append("\n"); }
								sb.append(String.format("%s : %s", 
										charField.name(), interpreter.interpret(this)));
							} catch (InstantiationException e) {
								throw new IllegalStateException(e);
							} catch (IllegalAccessException e) {
								throw new IllegalStateException(e);
							}
						}
					}
				}
			}
		}

		return sb.toString();
	}
}
