package tdanford.sunlight.cgp;

import java.util.LinkedList;

/**
 * MARC21 Documentation: 
 * http://www.loc.gov/marc/96principl.html
 * 
 * also: 
 * http://www.loc.gov/marc/holdings/hdintro.html
 * 
 * 
 * 
 * @author Timothy Danford
 */
public class MARC21Record {

	public MARC21Leader leader;
	public LinkedList<ControlField> controlFields;
	public LinkedList<DataField> dataFields;
	
	public MARC21Record() { 
		leader = null;
		controlFields = new LinkedList<ControlField>();
		dataFields = new LinkedList<DataField>();
	}	
}

class MARC21Leader { 
	public String content;
	
	public MARC21Leader() { content = ""; }
}

class MARC21VariableField {
	
	public char[] tag;
	public String content;
	
	public MARC21VariableField(String t) { 
		tag = t.toCharArray();
		content = "";
	}
}

class ControlField extends MARC21VariableField {
	public ControlField(String tag) { 
		super(tag);
	}
}

class DataField extends MARC21VariableField { 
	public char[] indicators;
	public LinkedList<SubField> subfields;
	
	public DataField(String tag, char... inds) {
		super(tag);
		indicators = new char[inds.length];
		subfields = new LinkedList<SubField>();
	}
}

class SubField { 
	public char code;
	public String content;
	
	public SubField(char c) { 
		code = c;
		content = "";
	}
}
