package tdanford.sunlight.cgp;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import java.util.regex.*;

import javax.swing.text.MutableAttributeSet;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLEditorKit;

import org.xml.sax.SAXException;

public class SunlightCGPDownloader {
	
	public static String SunlightCGPData = "http://sunlightlabs.com/cgp-data/";

	public static void main(String[] args) { 
		File dir = new File(args[0]);
		try {
			Collection<MARC21Record> recs = new CGPParser(new SunlightCGPDirectory(dir)).getRecords();
			
			for(MARC21Record rec : recs) { 
				System.out.println(">> " + rec.leader.content);
				BibliographicRecord bibRec = new BibliographicRecord(rec);
				System.out.println(bibRec.interpret());
				System.out.println();
			}
			
			System.out.println(String.format("# Records: %d", recs.size()));
			
		} catch (SAXException e) {
			e.printStackTrace(System.err);
		} catch (IOException e) {
			e.printStackTrace(System.err);
		}
	}
	
	
}

class SunlightDirFinder extends HTMLEditorKit.ParserCallback { 
	
	private static Pattern dirPattern = Pattern.compile("^(\\d+)/?$");
	
	private URL base;
	private int depth;
	private File dir;
	
	private Map<String,SunlightDirFinder> subdirs;
	private SunlightXMLFinder finder;
	
	public SunlightDirFinder(File d, int dpth, URL b) {
		dir = d;
		base = b;
		depth = dpth;
	}

	public void handleStartTag(HTML.Tag t, 
			MutableAttributeSet a, int pos) { 

		if(t.equals(HTML.Tag.A)) { 
			String href = String.valueOf(a.getAttribute("href"));
			Matcher m = dirPattern.matcher(href);
			if(m.matches()) { 
				String dirName = m.group(1);
				
				if(depth > 0) {
					
					
				} else { 
					
				}
			}
		}
	}
}


class SunlightXMLFinder extends HTMLEditorKit.ParserCallback { 
	
	private Pattern xmlPattern = Pattern.compile("^(\\d+)-(\\d+)\\.xml$");

	private File dir;
	private URL base;
	private Map<String,Map<Integer,URL>> urls;
	
	public SunlightXMLFinder(File d, URL b) {
		dir = d;
		base = b;
		urls = new TreeMap<String,Map<Integer,URL>>();

		if(!dir.isDirectory() || !dir.canWrite()) { 
			throw new IllegalArgumentException(String.format("%s is not dir, or not write-able", 
					dir.getAbsolutePath()));
		}
	}
	
	public void handleStartTag(HTML.Tag t, 
			MutableAttributeSet a, int pos) { 

		if(t.equals(HTML.Tag.A)) { 
			String href = String.valueOf(a.getAttribute("href"));
			Matcher m = xmlPattern.matcher(href);
			if(m.matches()) { 
				String key = m.group(1); 
				Integer version = Integer.parseInt(m.group(2));
				if(!urls.containsKey(key)) { urls.put(key, new TreeMap<Integer,URL>()); }
				if(urls.get(key).containsKey(version)) { 
					throw new IllegalStateException(String.format("Duplicate key-version %s-%d (%s vs. %s)",
							key, version, href, urls.get(key).get(version).toString()));
				}
				
				try {
					urls.get(key).put(version, new URL(base, href));
					
				} catch (MalformedURLException e) {
					throw new IllegalArgumentException(String.format(
							"Illegal base %s + href %s URL", 
							base.toString(), href));
				}
			}
		}
	}
}

